@@ -32,8 +32,16 @@ static int days_in_month[13]      = {  31,  31,  28,  31,  30,  31,  30,  31,  3
 static void do_range_limit(timelib_sll start, timelib_sll end, timelib_sll adj, timelib_sll *a, timelib_sll *b)
 {
 	if (*a < start) {
-		*b -= (start - *a - 1) / adj + 1;
-		*a += adj * ((start - *a - 1) / adj + 1);
+		/* We calculate 'a + 1' first as 'start - *a - 1' causes an int64_t overflows if *a is
+		 * LONG_MIN. 'start' is 0 in this context, and '0 - LONG_MIN > LONG_MAX'. */
+		timelib_sll a_plus_1 = *a + 1;
+
+		*b -= (start - a_plus_1) / adj + 1;
+
+		/* This code add the extra 'adj' separately, as otherwise this can overflow int64_t in
+		 * situations where *b is near LONG_MIN. */
+		*a += adj * ((start - a_plus_1) / adj);
+		*a += adj;
 	}
 	if (*a >= end) {
 		*b += *a / adj;
@@ -104,9 +112,10 @@ static void do_range_limit_days_relative(timelib_sll *base_y, timelib_sll *base_
 static int do_range_limit_days(timelib_sll *y, timelib_sll *m, timelib_sll *d)
 {
 	timelib_sll leapyear;
-	timelib_sll days_this_month;
-	timelib_sll last_month, last_year;
-	timelib_sll days_last_month;
+	timelib_sll previous_month, previous_year;
+	timelib_sll days_in_previous_month;
+	int retval = 0;
+	int *days_per_month_current_year;
 
 	/* can jump an entire leap year period quickly */
 	if (*d >= DAYS_PER_ERA || *d <= -DAYS_PER_ERA) {
@@ -117,29 +126,29 @@ static int do_range_limit_days(timelib_sll *y, timelib_sll *m, timelib_sll *d)
 	do_range_limit(1, 13, 12, m, y);
 
 	leapyear = timelib_is_leap(*y);
-	days_this_month = leapyear ? days_in_month_leap[*m] : days_in_month[*m];
-	last_month = (*m) - 1;
+	days_per_month_current_year = leapyear ? days_in_month_leap : days_in_month;
 
-	if (last_month < 1) {
-		last_month += 12;
-		last_year = (*y) - 1;
-	} else {
-		last_year = (*y);
-	}
-	leapyear = timelib_is_leap(last_year);
-	days_last_month = leapyear ? days_in_month_leap[last_month] : days_in_month[last_month];
+	while (*d <= 0 && *m > 0) {
+		previous_month = (*m) - 1;
+		if (previous_month < 1) {
+			previous_month += 12;
+			previous_year = (*y) - 1;
+		} else {
+			previous_year = (*y);
+		}
+		leapyear = timelib_is_leap(previous_year);
+		days_in_previous_month = leapyear ? days_in_month_leap[previous_month] : days_in_month[previous_month];
 
-	if (*d <= 0) {
-		*d += days_last_month;
+		*d += days_in_previous_month;
 		(*m)--;
-		return 1;
+		retval = 1;
 	}
-	if (*d > days_this_month) {
-		*d -= days_this_month;
+	while (*d > 0 && *m <= 12 && *d > days_per_month_current_year[*m]) {
+		*d -=  days_per_month_current_year[*m];
 		(*m)++;
-		return 1;
+		retval = 1;
 	}
-	return 0;
+	return retval;
 }
 
 static void do_adjust_for_weekday(timelib_time* time)
@@ -376,7 +385,11 @@ static void do_adjust_timezone(timelib_time *tz, timelib_tzinfo *tzi)
 
 		default: {
 			/* No timezone in struct, fallback to reference if possible */
-			timelib_time_offset *current, *after;
+			int32_t              current_offset = 0;
+			timelib_sll          current_transition_time = 0;
+			unsigned int         current_is_dst = 0;
+			int32_t              after_offset = 0;
+			timelib_sll          after_transition_time = 0;
 			timelib_sll          adjustment;
 			int                  in_transition;
 			int32_t              actual_offset;
@@ -386,49 +399,48 @@ static void do_adjust_timezone(timelib_time *tz, timelib_tzinfo *tzi)
 				return;
 			}
 
-			current = timelib_get_time_zone_info(tz->sse, tzi);
-			after = timelib_get_time_zone_info(tz->sse - current->offset, tzi);
-			actual_offset = after->offset;
-			actual_transition_time = after->transition_time;
-			if (current->offset == after->offset && tz->have_zone) {
+			timelib_get_time_zone_offset_info(tz->sse, tzi, &current_offset, &current_transition_time, &current_is_dst);
+			timelib_get_time_zone_offset_info(tz->sse - current_offset, tzi, &after_offset, &after_transition_time, NULL);
+			actual_offset = after_offset;
+			actual_transition_time = after_transition_time;
+			if (current_offset == after_offset && tz->have_zone) {
 				/* Make sure we're not missing a DST change because we don't know the actual offset yet */
-				if (current->offset >= 0 && tz->dst && !current->is_dst) {
+				if (current_offset >= 0 && tz->dst && !current_is_dst) {
 						/* Timezone or its DST at or east of UTC, so the local time, interpreted as UTC, leaves DST (as defined in the actual timezone) before the actual local time */
-						timelib_time_offset *earlier;
-						earlier = timelib_get_time_zone_info(tz->sse - current->offset - 7200, tzi);
-						if ((earlier->offset != after->offset) && (tz->sse - earlier->offset < after->transition_time)) {
+						int32_t              earlier_offset;
+						timelib_sll          earlier_transition_time;
+						timelib_get_time_zone_offset_info(tz->sse - current_offset - 7200, tzi, &earlier_offset, &earlier_transition_time, NULL);
+						if ((earlier_offset != after_offset) && (tz->sse - earlier_offset < after_transition_time)) {
 								/* Looking behind a bit clarified the actual offset to use */
-								actual_offset = earlier->offset;
-								actual_transition_time = earlier->transition_time;
+								actual_offset = earlier_offset;
+								actual_transition_time = earlier_transition_time;
 						}
-						timelib_time_offset_dtor(earlier);
-				} else if (current->offset <= 0 && current->is_dst && !tz->dst) {
+				} else if (current_offset <= 0 && current_is_dst && !tz->dst) {
 						/* Timezone west of UTC, so the local time, interpreted as UTC, leaves DST (as defined in the actual timezone) after the actual local time */
-						timelib_time_offset *later;
-						later = timelib_get_time_zone_info(tz->sse - current->offset + 7200, tzi);
-						if ((later->offset != after->offset) && (tz->sse - later->offset >= later->transition_time)) {
+						int32_t              later_offset;
+						timelib_sll          later_transition_time;
+						timelib_get_time_zone_offset_info(tz->sse - current_offset + 7200, tzi, &later_offset, &later_transition_time, NULL);
+						if ((later_offset != after_offset) && (tz->sse - later_offset >= later_transition_time)) {
 								/* Looking ahead a bit clarified the actual offset to use */
-								actual_offset = later->offset;
-								actual_transition_time = later->transition_time;
+								actual_offset = later_offset;
+								actual_transition_time = later_transition_time;
 						}
-						timelib_time_offset_dtor(later);
 				}
 			}
 
 			tz->is_localtime = 1;
 
 			in_transition = (
-				((tz->sse - actual_offset) >= (actual_transition_time + (current->offset - actual_offset))) &&
+				actual_transition_time != INT64_MIN &&
+				((tz->sse - actual_offset) >= (actual_transition_time + (current_offset - actual_offset))) &&
 				((tz->sse - actual_offset) < actual_transition_time)
 			);
 
-			if ((current->offset != actual_offset) && !in_transition) {
+			if ((current_offset != actual_offset) && !in_transition) {
 				adjustment = -actual_offset;
 			} else {
-				adjustment = -current->offset;
+				adjustment = -current_offset;
 			}
-			timelib_time_offset_dtor(current);
-			timelib_time_offset_dtor(after);
 
 			tz->sse += adjustment;
 			timelib_set_timezone(tz, tzi);
@@ -458,9 +470,15 @@ void timelib_update_ts(timelib_time* time, timelib_tzinfo* tzi)
 	do_adjust_relative(time);
 	do_adjust_special(time);
 
-	time->sse =
-		(timelib_epoch_days_from_time(time) * SECS_PER_DAY) +
-		timelib_hms_to_seconds(time->h, time->i, time->s);
+	/* You might be wondering, why this code does this in two steps. This is because
+	 * timelib_epoch_days_from_time(time) * SECS_PER_DAY with the lowest limit of
+	 * timelib_epoch_days_from_time() is less than the range of an int64_t. This then overflows. In
+	 * order to be able to still allow for any time in that day that only halfly fits in the int64_t
+	 * range, we add the time element first, which is always positive, and then twice half the value
+	 * of the earliest day as expressed as unix timestamp. */
+	time->sse = timelib_hms_to_seconds(time->h, time->i, time->s);
+	time->sse += timelib_epoch_days_from_time(time) * (SECS_PER_DAY / 2);
+	time->sse += timelib_epoch_days_from_time(time) * (SECS_PER_DAY / 2);
 
 	// This modifies time->sse, if needed
 	do_adjust_timezone(time, tzi);
