@@ -275,6 +275,9 @@ interface DOMParentNode
 
     /** @param DOMNode|string $nodes */
     public function prepend(...$nodes): void;
+
+    /** @param DOMNode|string $nodes */
+    public function replaceChildren(...$nodes): void;
 }
 
 interface DOMChildNode
@@ -304,6 +307,9 @@ class DOMNode
     /** @readonly */
     public ?DOMNode $parentNode;
 
+    /** @readonly */
+    public ?DOMElement $parentElement;
+
     /** @readonly */
     public DOMNodeList $childNodes;
 
@@ -322,6 +328,9 @@ class DOMNode
     /** @readonly */
     public ?DOMNamedNodeMap $attributes;
 
+    /** @readonly */
+    public bool $isConnected;
+
     /** @readonly */
     public ?DOMDocument $ownerDocument;
 
@@ -375,6 +384,8 @@ class DOMNode
     /** @tentative-return-type */
     public function isSameNode(DOMNode $otherNode): bool {}
 
+    public function isEqualNode(?DOMNode $otherNode): bool {}
+
     /** @tentative-return-type */
     public function isSupported(string $feature, string $version): bool {}
 
@@ -392,6 +403,10 @@ class DOMNode
 
     /** @return DOMNode|false */
     public function replaceChild(DOMNode $node, DOMNode $child) {}
+
+    public function contains(DOMNode|DOMNameSpaceNode|null $other): bool {}
+
+    public function getRootNode(?array $options = null): DOMNode {}
 }
 
 class DOMNameSpaceNode
@@ -414,12 +429,18 @@ class DOMNameSpaceNode
     /** @readonly */
     public ?string $namespaceURI;
 
+    /** @readonly */
+    public bool $isConnected;
+
     /** @readonly */
     public ?DOMDocument $ownerDocument;
 
     /** @readonly */
     public ?DOMNode $parentNode;
 
+    /** @readonly */
+    public ?DOMElement $parentElement;
+
     /** @implementation-alias DOMNode::__sleep */
     public function __sleep(): array {}
 
@@ -463,6 +484,9 @@ class DOMDocumentFragment extends DOMNode implements DOMParentNode
 
     /** @param DOMNode|string $nodes */
     public function prepend(...$nodes): void {}
+
+    /** @param DOMNode|string $nodes */
+    public function replaceChildren(...$nodes): void {}
 }
 
 class DOMNodeList implements IteratorAggregate, Countable
@@ -493,7 +517,7 @@ class DOMCharacterData extends DOMNode implements DOMChildNode
     public ?DOMElement $nextElementSibling;
 
     /** @tentative-return-type */
-    public function appendData(string $data): bool {}
+    public function appendData(string $data): true {}
 
     /** @return string|false */
     public function substringData(int $offset, int $count) {}
@@ -546,6 +570,10 @@ class DOMElement extends DOMNode implements DOMParentNode, DOMChildNode
     /** @readonly */
     public string $tagName;
 
+    public string $className;
+
+    public string $id;
+
     /** @readonly */
     public mixed $schemaTypeInfo = null;
 
@@ -569,6 +597,8 @@ class DOMElement extends DOMNode implements DOMParentNode, DOMChildNode
     /** @tentative-return-type */
     public function getAttribute(string $qualifiedName): string {}
 
+    public function getAttributeNames(): array {}
+
     /** @tentative-return-type */
     public function getAttributeNS(?string $namespace, string $localName): string {}
 
@@ -620,6 +650,8 @@ class DOMElement extends DOMNode implements DOMParentNode, DOMChildNode
     /** @tentative-return-type */
     public function setIdAttributeNode(DOMAttr $attr, bool $isId): void {}
 
+    public function toggleAttribute(string $qualifiedName, ?bool $force = null): bool {}
+
     public function remove(): void {}
 
     /** @param DOMNode|string $nodes */
@@ -636,6 +668,13 @@ class DOMElement extends DOMNode implements DOMParentNode, DOMChildNode
 
     /** @param DOMNode|string $nodes */
     public function prepend(...$nodes): void {}
+
+    /** @param DOMNode|string $nodes */
+    public function replaceChildren(...$nodes): void {}
+
+    public function insertAdjacentElement(string $where, DOMElement $element): ?DOMElement {}
+
+    public function insertAdjacentText(string $where, string $data): void {}
 }
 
 class DOMDocument extends DOMNode implements DOMParentNode
@@ -743,11 +782,11 @@ class DOMDocument extends DOMNode implements DOMParentNode
     /** @return DOMNode|false */
     public function importNode(DOMNode $node, bool $deep = false) {}
 
-    /** @return bool */
-    public function load(string $filename, int $options = 0) {}
+    /** @tentative-return-type */
+    public function load(string $filename, int $options = 0): bool {}
 
-    /** @return bool */
-    public function loadXML(string $source, int $options = 0) {}
+    /** @tentative-return-type */
+    public function loadXML(string $source, int $options = 0): bool {}
 
     /** @tentative-return-type */
     public function normalizeDocument(): void {}
@@ -759,11 +798,11 @@ class DOMDocument extends DOMNode implements DOMParentNode
     public function save(string $filename, int $options = 0): int|false {}
 
 #ifdef LIBXML_HTML_ENABLED
-    /** @return bool */
-    public function loadHTML(string $source, int $options = 0) {}
+    /** @tentative-return-type */
+    public function loadHTML(string $source, int $options = 0): bool {}
 
-    /** @return bool */
-    public function loadHTMLFile(string $filename, int $options = 0) {}
+    /** @tentative-return-type */
+    public function loadHTMLFile(string $filename, int $options = 0): bool {}
 
     /** @tentative-return-type */
     public function saveHTML(?DOMNode $node = null): string|false {}
@@ -795,14 +834,17 @@ class DOMDocument extends DOMNode implements DOMParentNode
     /** @tentative-return-type */
     public function xinclude(int $options = 0): int|false {}
 
-    /** @return DOMNode|false */
-    public function adoptNode(DOMNode $node) {}
+    /** @tentative-return-type */
+    public function adoptNode(DOMNode $node): DOMNode|false {}
 
     /** @param DOMNode|string $nodes */
     public function append(...$nodes): void {}
 
     /** @param DOMNode|string $nodes */
     public function prepend(...$nodes): void {}
+
+    /** @param DOMNode|string $nodes */
+    public function replaceChildren(...$nodes): void {}
 }
 
 final class DOMException extends Exception
@@ -933,4 +975,4 @@ class DOMXPath
 }
 #endif
 
-function dom_import_simplexml(object $node): DOMAttr|DOMElement {}
+function dom_import_simplexml(object $node): DOMElement {}
