@@ -39,30 +39,8 @@
 #include "mbfilter_pass.h"
 #include "mbfilter_8bit.h"
 
-#include "filters/mbfilter_euc_cn.h"
-#include "filters/mbfilter_hz.h"
-#include "filters/mbfilter_euc_tw.h"
-#include "filters/mbfilter_big5.h"
-#include "filters/mbfilter_uhc.h"
-#include "filters/mbfilter_euc_kr.h"
-#include "filters/mbfilter_iso2022_kr.h"
-#include "filters/mbfilter_sjis.h"
-#include "filters/mbfilter_sjis_mobile.h"
-#include "filters/mbfilter_sjis_mac.h"
-#include "filters/mbfilter_sjis_2004.h"
-#include "filters/mbfilter_cp51932.h"
-#include "filters/mbfilter_jis.h"
-#include "filters/mbfilter_iso2022_jp_ms.h"
-#include "filters/mbfilter_iso2022jp_2004.h"
-#include "filters/mbfilter_iso2022jp_mobile.h"
-#include "filters/mbfilter_euc_jp.h"
-#include "filters/mbfilter_euc_jp_win.h"
-#include "filters/mbfilter_euc_jp_2004.h"
-#include "filters/mbfilter_gb18030.h"
-#include "filters/mbfilter_cp932.h"
-#include "filters/mbfilter_cp936.h"
-#include "filters/mbfilter_cp5022x.h"
 #include "filters/mbfilter_base64.h"
+#include "filters/mbfilter_cjk.h"
 #include "filters/mbfilter_qprint.h"
 #include "filters/mbfilter_uuencode.h"
 #include "filters/mbfilter_7bit.h"
@@ -212,12 +190,6 @@ const mbfl_encoding *mbfl_no2encoding(enum mbfl_no_encoding no_encoding)
 	return NULL;
 }
 
-enum mbfl_no_encoding mbfl_name2no_encoding(const char *name)
-{
-	const mbfl_encoding *encoding = mbfl_name2encoding(name);
-	return encoding ? encoding->no_encoding : mbfl_no_encoding_invalid;
-}
-
 const char *mbfl_no_encoding2name(enum mbfl_no_encoding no_encoding)
 {
 	const mbfl_encoding *encoding = mbfl_no2encoding(no_encoding);
@@ -229,11 +201,6 @@ const mbfl_encoding **mbfl_get_supported_encodings(void)
 	return mbfl_encoding_ptr_list;
 }
 
-const char *mbfl_no2preferred_mime_name(enum mbfl_no_encoding no_encoding)
-{
-	return mbfl_encoding_preferred_mime_name(mbfl_no2encoding(no_encoding));
-}
-
 const char *mbfl_encoding_preferred_mime_name(const mbfl_encoding *encoding)
 {
 	if (encoding->mime_name && encoding->mime_name[0] != '\0') {
