@@ -1,5 +1,5 @@
 /* This is a generated file, edit the .stub.php file instead.
- * Stub hash: d28c6b566f4739b1eaefb82032e620aeb59728dc */
+ * Stub hash: a64427da8198261f528a38c5bf90c673eb5b8282 */
 
 ZEND_BEGIN_ARG_WITH_RETURN_TYPE_MASK_EX(arginfo_socket_select, 0, 4, MAY_BE_LONG|MAY_BE_FALSE)
 	ZEND_ARG_TYPE_INFO(1, read, IS_ARRAY, 1)
@@ -140,6 +140,12 @@ ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(arginfo_socket_shutdown, 0, 1, _IS_BOOL,
 ZEND_END_ARG_INFO()
 #endif
 
+#if defined(HAVE_SOCKATMARK)
+ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(arginfo_socket_atmark, 0, 1, _IS_BOOL, 0)
+	ZEND_ARG_OBJ_INFO(0, socket, Socket, 0)
+ZEND_END_ARG_INFO()
+#endif
+
 ZEND_BEGIN_ARG_WITH_RETURN_TYPE_INFO_EX(arginfo_socket_last_error, 0, 0, IS_LONG, 0)
 	ZEND_ARG_OBJ_INFO_WITH_DEFAULT_VALUE(0, socket, Socket, 1, "null")
 ZEND_END_ARG_INFO()
@@ -237,6 +243,9 @@ ZEND_FUNCTION(socket_create_pair);
 #if defined(HAVE_SHUTDOWN)
 ZEND_FUNCTION(socket_shutdown);
 #endif
+#if defined(HAVE_SOCKATMARK)
+ZEND_FUNCTION(socket_atmark);
+#endif
 ZEND_FUNCTION(socket_last_error);
 ZEND_FUNCTION(socket_clear_error);
 ZEND_FUNCTION(socket_import_stream);
@@ -288,6 +297,9 @@ static const zend_function_entry ext_functions[] = {
 #endif
 #if defined(HAVE_SHUTDOWN)
 	ZEND_FE(socket_shutdown, arginfo_socket_shutdown)
+#endif
+#if defined(HAVE_SOCKATMARK)
+	ZEND_FE(socket_atmark, arginfo_socket_atmark)
 #endif
 	ZEND_FE(socket_last_error, arginfo_socket_last_error)
 	ZEND_FE(socket_clear_error, arginfo_socket_clear_error)
@@ -328,6 +340,9 @@ static void register_sockets_symbols(int module_number)
 	REGISTER_LONG_CONSTANT("AF_INET", AF_INET, CONST_PERSISTENT);
 #if HAVE_IPV6
 	REGISTER_LONG_CONSTANT("AF_INET6", AF_INET6, CONST_PERSISTENT);
+#endif
+#if defined(AF_DIVERT)
+	REGISTER_LONG_CONSTANT("AF_DIVERT", AF_DIVERT, CONST_PERSISTENT);
 #endif
 	REGISTER_LONG_CONSTANT("SOCK_STREAM", SOCK_STREAM, CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("SOCK_DGRAM", SOCK_DGRAM, CONST_PERSISTENT);
@@ -376,6 +391,9 @@ static void register_sockets_symbols(int module_number)
 	REGISTER_LONG_CONSTANT("SO_REUSEADDR", SO_REUSEADDR, CONST_PERSISTENT);
 #if defined(SO_REUSEPORT)
 	REGISTER_LONG_CONSTANT("SO_REUSEPORT", SO_REUSEPORT, CONST_PERSISTENT);
+#endif
+#if defined(SO_REUSEPORT_LB)
+	REGISTER_LONG_CONSTANT("SO_REUSEPORT_LB", SO_REUSEPORT_LB, CONST_PERSISTENT);
 #endif
 	REGISTER_LONG_CONSTANT("SO_KEEPALIVE", SO_KEEPALIVE, CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("SO_DONTROUTE", SO_DONTROUTE, CONST_PERSISTENT);
@@ -417,6 +435,15 @@ static void register_sockets_symbols(int module_number)
 #if defined(SO_ACCEPTFILTER)
 	REGISTER_LONG_CONSTANT("SO_ACCEPTFILTER", SO_ACCEPTFILTER, CONST_PERSISTENT);
 #endif
+#if defined(SO_RERROR)
+	REGISTER_LONG_CONSTANT("SO_RERROR", SO_RERROR, CONST_PERSISTENT);
+#endif
+#if defined(SO_SOPLICE)
+	REGISTER_LONG_CONSTANT("SO_SPLICE", SO_SPLICE, CONST_PERSISTENT);
+#endif
+#if defined(SO_ZEROIZE)
+	REGISTER_LONG_CONSTANT("SO_ZEROIZE", SO_ZEROIZE, CONST_PERSISTENT);
+#endif
 #if defined(SOL_FILTER)
 	REGISTER_LONG_CONSTANT("SOL_FILTER", SOL_FILTER, CONST_PERSISTENT);
 #endif
@@ -549,6 +576,9 @@ static void register_sockets_symbols(int module_number)
 	REGISTER_LONG_CONSTANT("IP_MULTICAST_IF", IP_MULTICAST_IF, CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("IP_MULTICAST_TTL", IP_MULTICAST_TTL, CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("IP_MULTICAST_LOOP", IP_MULTICAST_LOOP, CONST_PERSISTENT);
+#if defined(IP_BIND_ADDRESS_NO_PORT)
+	REGISTER_LONG_CONSTANT("IP_BIND_ADDRESS_NO_PORT", IP_BIND_ADDRESS_NO_PORT, CONST_PERSISTENT);
+#endif
 #if HAVE_IPV6
 	REGISTER_LONG_CONSTANT("IPV6_MULTICAST_IF", IPV6_MULTICAST_IF, CONST_PERSISTENT);
 #endif
@@ -900,6 +930,9 @@ static void register_sockets_symbols(int module_number)
 #endif
 	REGISTER_LONG_CONSTANT("SOL_TCP", IPPROTO_TCP, CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("SOL_UDP", IPPROTO_UDP, CONST_PERSISTENT);
+#if defined(IPPROTO_UDPLITE)
+	REGISTER_LONG_CONSTANT("SOL_UDPLITE", IPPROTO_UDPLITE, CONST_PERSISTENT);
+#endif
 #if HAVE_IPV6
 	REGISTER_LONG_CONSTANT("IPV6_UNICAST_HOPS", IPV6_UNICAST_HOPS, CONST_PERSISTENT);
 #endif
@@ -967,6 +1000,51 @@ static void register_sockets_symbols(int module_number)
 #if (!defined(LOCAL_CREDS_PERSISTENT) && defined(LOCAL_CREDS))
 	REGISTER_LONG_CONSTANT("LOCAL_CREDS", LOCAL_CREDS, CONST_PERSISTENT);
 #endif
+#if defined(SO_ATTACH_REUSEPORT_CBPF)
+	REGISTER_LONG_CONSTANT("SO_ATTACH_REUSEPORT_CBPF", SO_ATTACH_REUSEPORT_CBPF, CONST_PERSISTENT);
+#endif
+#if defined(SO_DETACH_FILTER)
+	REGISTER_LONG_CONSTANT("SO_DETACH_FILTER", SO_DETACH_FILTER, CONST_PERSISTENT);
+#endif
+#if defined(SO_DETACH_BPF)
+	REGISTER_LONG_CONSTANT("SO_DETACH_BPF", SO_DETACH_BPF, CONST_PERSISTENT);
+#endif
+#if defined(TCP_QUICKACK)
+	REGISTER_LONG_CONSTANT("TCP_QUICKACK", TCP_QUICKACK, CONST_PERSISTENT);
+#endif
+#if defined(TCP_REPAIR)
+	REGISTER_LONG_CONSTANT("TCP_REPAIR", TCP_REPAIR, CONST_PERSISTENT);
+#endif
+#if defined(IP_DONTFRAG)
+	REGISTER_LONG_CONSTANT("IP_DONTFRAG", IP_DONTFRAG, CONST_PERSISTENT);
+#endif
+#if defined(IP_MTU_DISCOVER)
+	REGISTER_LONG_CONSTANT("IP_MTU_DISCOVER", IP_MTU_DISCOVER, CONST_PERSISTENT);
+#endif
+#if defined(IP_PMTUDISC_DO)
+	REGISTER_LONG_CONSTANT("IP_PMTUDISC_DO", IP_PMTUDISC_DO, CONST_PERSISTENT);
+#endif
+#if defined(IP_PMTUDISC_DONT)
+	REGISTER_LONG_CONSTANT("IP_PMTUDISC_DONT", IP_PMTUDISC_DONT, CONST_PERSISTENT);
+#endif
+#if defined(IP_PMTUDISC_WANT)
+	REGISTER_LONG_CONSTANT("IP_PMTUDISC_WANT", IP_PMTUDISC_WANT, CONST_PERSISTENT);
+#endif
+#if defined(IP_PMTUDISC_PROBE)
+	REGISTER_LONG_CONSTANT("IP_PMTUDISC_PROBE", IP_PMTUDISC_PROBE, CONST_PERSISTENT);
+#endif
+#if defined(IP_PMTUDISC_INTERFACE)
+	REGISTER_LONG_CONSTANT("IP_PMTUDISC_INTERFACE", IP_PMTUDISC_INTERFACE, CONST_PERSISTENT);
+#endif
+#if defined(IP_PMTUDISC_OMIT)
+	REGISTER_LONG_CONSTANT("IP_PMTUDISC_OMIT", IP_PMTUDISC_OMIT, CONST_PERSISTENT);
+#endif
+#if defined(UDPLITE_SEND_CSCOV)
+	REGISTER_LONG_CONSTANT("UDPLITE_SEND_CSCOV", UDPLITE_SEND_CSCOV, CONST_PERSISTENT);
+#endif
+#if defined(UDPLITE_RECV_CSCOV)
+	REGISTER_LONG_CONSTANT("UDPLITE_RECV_CSCOV", UDPLITE_RECV_CSCOV, CONST_PERSISTENT);
+#endif
 }
 
 static zend_class_entry *register_class_Socket(void)
